//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

/*
	FIXME!
	There are a lot of things to optimize in this class ex:

	* I create new Vectors, BoundingBox2 and other objects when i could pre-compute it
	* There are a few number constants that should be declaired as constants
	* Screen width and height needs to be GLOBAL!
*/

package starship2d;

import org.lwjgl.Display;
import org.lwjgl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.Window;
import org.lwjgl.opengl.glu.Cylinder;
import org.lwjgl.opengl.glu.Disk;
import org.lwjgl.opengl.glu.GLU;
import org.lwjgl.opengl.glu.PartialDisk;
import org.lwjgl.opengl.glu.Sphere;
import org.lwjgl.input.Keyboard;

public class Enemy extends Object
{
	// VARIABLES
	// FIXME! these should be dependent on the timers resolution
	private static final float BULLET_REPEAT = 5000000.0f;
	private static final float BULLET_SPEED = 0.5f;
	private static final int NUM_BULLETS = 5;
	private static final int ENEMY_SCORE = 100;

	private int scrWidth = 640;
	private int scrHeight = 480;

	private int txtRocket, txtMask;
	private Vector2 pos = new Vector2(100,400);
	private Vector2 v = new Vector2(0.0f,-0.03f);
	private Vector2 size =  new Vector2(40,40);
	private ParticleExplosion pExplosion = new ParticleExplosion(30);
	private boolean isHit = false;
	private Layer layer;
	private Player player;
	private float sqrRadius = 0;
	private float rot = 0;
	private int numBullets = 0;
	private long lastBulTime = 0;
	private Particle bullets[] = new Particle[NUM_BULLETS];

	// FUNCTIONS
	public Enemy(Layer lay, Player pl, String path, String maskPath)
	{
		layer = lay;
		player = pl;

		txtRocket = Texture.loadTexture(path);
		txtMask = Texture.loadTexture(maskPath);

		sqrRadius = size.x*size.x / 4 + size.y*size.y / 4;

		for (int i=0; i < NUM_BULLETS; i++)
		{
			bullets[i] = new Particle("starship2d/Data/particle.png",new Vector2(-100,-100), new Vector2(20.0f,20.0f),
										new Vector2(0,0), new Vector3(1.0f,1.0f,0), 0.9f, 0.0f);
		}

		lastBulTime = (long) (Math.random() * 2 * BULLET_REPEAT);
	}

	public Vector2 getSize()					{ return new Vector2(size); }
	public Vector2 getPos()						{ return new Vector2(pos); }
	public void setVelocity(Vector2 vel)		{ v = new Vector2(vel.x,vel.y); }
	public void setPos(Vector2 p)				{ pos =  new Vector2(p.x,p.y); }
	public void setSize(Vector2 s)
	{
		size =  new Vector2(s.x,s.y);
		sqrRadius = size.x*size.x / 4 + size.y*size.y / 4;
	}

	public void init()
	{
	}

	// return false if the it should be destroyd
	public boolean render()
	{
		boolean res = true;

		GL11.glEnable(GL11.GL_BLEND);

		if (isHit)
		{
			GL11.glBlendFunc(GL11.GL_SRC_ALPHA,GL11.GL_ONE);
			res =  pExplosion.render();
		}
		else
		{
			GL11.glPushMatrix();
			GL11.glLoadIdentity();

			GL11.glTranslatef(pos.x + size.x/2,pos.y + size.y/2,0);
			GL11.glRotatef(rot,0,0,1);

			// draw the mask
			GL11.glBlendFunc(GL11.GL_DST_COLOR,GL11.GL_ZERO);
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, txtMask);
			GL11.glBegin(GL11.GL_QUADS);
				GL11.glTexCoord2f(0,0);		GL11.glVertex2f(-size.x/2, -size.y/2);
				GL11.glTexCoord2f(0,-1);	GL11.glVertex2f(-size.x/2, size.y/2);
				GL11.glTexCoord2f(-1,-1);	GL11.glVertex2f(size.x/2, size.y/2);
				GL11.glTexCoord2f(-1,0);	GL11.glVertex2f(size.x/2, -size.y/2);
			GL11.glEnd();

			// draw the texture
			GL11.glBlendFunc(GL11.GL_ONE, GL11.GL_ONE);
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, txtRocket);
			GL11.glBegin(GL11.GL_QUADS);
				GL11.glTexCoord2f(0,0);		GL11.glVertex2f(-size.x/2, -size.y/2);
				GL11.glTexCoord2f(0,-1);	GL11.glVertex2f(-size.x/2, size.y/2);
				GL11.glTexCoord2f(-1,-1);	GL11.glVertex2f(size.x/2, size.y/2);
				GL11.glTexCoord2f(-1,0);	GL11.glVertex2f(size.x/2, -size.y/2);
			GL11.glEnd();

			GL11.glPopMatrix();

			// remove the enemy if its belove the screen
			if (pos.y > scrHeight)
				res = false;
			else
				res = true;
		}

		// render the bullets
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA,GL11.GL_ONE);
		for (int i=0; i < numBullets; i++)
		{
			bullets[i].render();
		}
		// keep rendering if there are bullets left, even if the enemy is killed
		res = (numBullets > 0) || res;

		GL11.glDisable(GL11.GL_BLEND);

		return res;
	}

	public void update()
	{
		if (!isHit)
		{
			rot += 100*Timer.frameInterval;
			pos.x += v.x * (Timer.frameInterval * 500);
			pos.y += v.y * (Timer.frameInterval * 500);
		}
		
		// fire bullet
		if (!isHit && (pos.y > 0 && pos.y < scrHeight) && ((Timer.frameTime - lastBulTime) > BULLET_REPEAT)
			&& (numBullets <  NUM_BULLETS))
		{
			lastBulTime = Timer.frameTime;
			bullets[numBullets].setPos(pos);

			Vector2 vp = player.getPos();
			Vector2 vv = new Vector2(vp.x - pos.x, vp.y - pos.y);
			vv.normalize();
			vv.scale(BULLET_SPEED);

			bullets[numBullets].setVelocity(vv);

			// this if statement should always be true but just in case
			if (numBullets + 1 <= NUM_BULLETS)
				numBullets++;
			else
				System.out.println("ERROR bullets out of bounds");
		}

 		// update bullets
		for (int i=0; i < numBullets; i++)
		{
			Vector2 p = bullets[i].getPos();
			Vector2 s = bullets[i].getSize();

			BoundingBox2 bb = new BoundingBox2(p,s);
			if ((p.y < 0 || p.y > scrHeight || p.x < 0 || p.x > scrWidth) || player.hit(bb))
			{
				Particle b = bullets[i];
				bullets[i] = bullets[numBullets-1];
				bullets[numBullets-1] = b;

				numBullets--;
			}

		}
}

	public void destroy() {}

	public boolean hit(BoundingBox2 bb)
	{
		if (isHit)
			return false;

		// FIXME! optimize this, don't create a new BoundingBox2 instance every time
		BoundingBox2 bb2 = new BoundingBox2(pos,size);
		isHit = bb.boxInBox(new BoundingBox2(pos,size));

		if (isHit)
		{
			pExplosion.init(1.0f, 0.5f, 0.1f, 0, new Vector2(pos.x + size.x/2 - 5, pos.y + size.y/2 - 5),
							new Vector2(20,20), v);
			
			// Add score
			player.addScore(ENEMY_SCORE);
		}

		return isHit;
	}

	public boolean sphereHit(float sqrRad, Vector2 center)
	{
		if (isHit)
			return false;

		// FIXME! optimze this
		Vector2 vv = new Vector2( (pos.x + size.x / 2) - center.x, (pos.y + size.y / 2) - center.y);

		float dist = vv.x*vv.x + vv.y*vv.y;

		isHit = dist < sqrRad + sqrRadius;

		if (isHit)
		{
			pExplosion.init(1.0f, 0.5f, 0.1f, 0, new Vector2(pos.x + size.x/2 - 5, pos.y + size.y/2 - 5),
							new Vector2(20,20), v);
			
			player.addScore(ENEMY_SCORE);
		}
		return isHit;
	}

} // end Enemy
