//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

/*
	FIXME!
	There are a lot of things to optimize in this class ex:

	* I create new Vectors and other objects when i could pre-compute it
	* There are a few number constants that should be declaired as constants

	* Generating a new class with new enemy array every time
	a formation is regenerated is a unnessisary
*/

package starship2d;

public class EnemyFormations extends Object
{
	private static final int NUM_FORMATIONS = 2;
	private static final int NUM_ENEMIES = 6;

	private BaseFormation formations[];
	private Layer layer;
	private Player player;


	public EnemyFormations(Layer lay, Player pl)
	{
		layer = lay;
		player = pl;

		formations =  new BaseFormation[NUM_FORMATIONS];

		for (int i=0; i < NUM_FORMATIONS; i++)
		{
			generateFormation(i);
		}
	}

	public void init() {}

	// returning false means to be deleted
	public boolean render()
	{
		for (int i=0; i < NUM_FORMATIONS; i++)
		{
			if (!formations[i].render())
				generateFormation(i);
		}

		return true;
	}

	public void update()
	{
		for (int i=0; i < NUM_FORMATIONS; i++)
		{
			formations[i].update();
		}
	}

	public void destroy()
	{
	}

	public boolean hit(BoundingBox2 bb)
	{
		boolean res = false;

		for (int i=0; (i < NUM_FORMATIONS) && !res; i++)
		{
			res = formations[i].hit(bb);
		}

		return res;
	}

	public boolean sphereHit(float radius, Vector2 center)
	{
		boolean res = false;

		for (int i=0; i < NUM_FORMATIONS; i++)
		{
			res = formations[i].sphereHit(radius, center) || res;
		}

		return res;
	}

	private void generateFormation(int i)
	{
		// FIXME!
		// shouldn't hardcode this (3 is the number of formations there exists)
		// but i don't know if there exist a command like "enum" in java
		int rand = (int)(Math.random() * 100) % 3;

		switch (rand)
		{
			case 0:
			{
				formations[i] =  new SinusFormation(NUM_ENEMIES, new Vector2(Math.random()*560 + 40,-40), 300, 
													new Vector2(0.0f,0.1f), layer, player);
			}
			break;
			case 1:
			{
				formations[i] = new DotFormation(layer, player, NUM_ENEMIES, new Vector2(Math.random()*560 + 40,-40), 0.1f);
			}
			break;
			case 2:
			{
				formations[i] =  new RandomFormation(layer, player, NUM_ENEMIES, new Vector2(600,-40), new Vector2(20,-300), 0.1f);
			}
			break;
			default:
				System.out.println("ERORR");
			break;
		}
	}
} // end EnemyFormation


abstract class BaseFormation
{
	protected int numEnemies = 0;
	protected Enemy enemies[];
	protected Vector2 center;
	protected Vector2 v;
	protected Layer layer;
	protected Player player;

	// returning false will result in a regeneration of formation
	public abstract boolean render();

	public abstract void update();

	public abstract boolean hit(BoundingBox2 bb);
	public abstract boolean sphereHit(float radius, Vector2 center);
}

class SinusFormation extends BaseFormation
{
	private float amp = 0;

	public SinusFormation(int num, Vector2 vcenter, float amplitude, Vector2 vel, Layer lay, Player player)
	{
		layer = lay;

		center = new Vector2(vcenter);
		numEnemies = num;
		amp = amplitude;
		v = new Vector2(vel);

		float curY = vcenter.y;
		enemies = new Enemy[num];
		for (int i=0; i < num; i++)
		{
			enemies[i] = new Enemy(layer, player, "starship2d/Data/enblob.png", "starship2d/Data/enblobmask.png");
			enemies[i].setPos(new Vector2(center.x, curY));
			enemies[i].setVelocity(new Vector2(0,0));

			Vector2 size = enemies[i].getSize();
			curY -= size.y + 5;
		}
	}

	// returning false means to be deleted
	public boolean render()
	{
		boolean res = false;

		for (int i=0; i < numEnemies; i++)
		{
			res = enemies[i].render() || res;
		}

		return res;
	}

	// returning false will result in a regeneration of formation
	public void update()
	{
		float dx = v.x * (Timer.frameInterval * 500);
		float dy = v.y * (Timer.frameInterval * 500);

		Vector2 p;
		for (int i=0; i < numEnemies; i++)
		{
			p = enemies[i].getPos();
			p.x += dx;
			p.y += dy;

			enemies[i].setPos(new Vector2(p.x + Math.sin(p.y/(v.y*100))*amp*Timer.frameInterval, p.y));
			enemies[i].update();
		}

	}

	public boolean hit(BoundingBox2 bb)
	{
		boolean res = false;

		for (int i=0; (i < numEnemies) && !res; i++)
		{
			res = enemies[i].hit(bb);
		}

		return res;
	}

	public boolean sphereHit(float radius, Vector2 center)
	{
		boolean res = false;

		for (int i=0; i < numEnemies; i++)
		{
			res = enemies[i].sphereHit(radius, center) || res;
		}

		return res;
	}

} // end SinusFormation


class DotFormation extends BaseFormation
{
	public DotFormation(Layer lay, Player player, int num, Vector2 vcenter, float speed)
	{
		center = new Vector2(vcenter);
		layer = lay;
		numEnemies = num;
		enemies = new Enemy[num];

		float s = (float)Math.random()*speed + speed;
		for (int i=0; i < numEnemies; i++)
		{
			enemies[i] = new Enemy(layer, player, "starship2d/Data/ensun.png", "starship2d/Data/ensunmask.png");
			enemies[i].setPos(new Vector2(center));
			enemies[i].setVelocity(new Vector2(0,s));
		}
	}

	// returning false means to be deleted
	public boolean render()
	{
		boolean res = false;

		for (int i=0; i < numEnemies; i++)
		{
			res = enemies[i].render() || res;
		}

		return res;
	}

	// returning false will result in a regeneration of formation
	public void update()
	{
		for (int i=0; i < numEnemies; i++)
		{
			enemies[i].update();
		}
	}

	public boolean hit(BoundingBox2 bb)
	{
		boolean res = false;

		for (int i=0; (i < numEnemies) && !res; i++)
		{
			res = enemies[i].hit(bb);
		}

		return res;
	}

	public boolean sphereHit(float radius, Vector2 center)
	{
		boolean res = false;

		for (int i=0; i < numEnemies; i++)
		{
			res = enemies[i].sphereHit(radius, center) || res;
		}

		return res;
	}

} // end DotFormation


class RandomFormation extends BaseFormation
{
	public RandomFormation(Layer lay, Player player, int num, Vector2 max, Vector2 min, float speed)
	{
		layer = lay;
		numEnemies = num;
		enemies = new Enemy[num];

		for (int i=0; i < numEnemies; i++)
		{
			enemies[i] = new Enemy(layer, player, "starship2d/Data/ensplat.png", "starship2d/Data/ensplatmask.png");
			enemies[i].setPos(new Vector2(Math.random()*(max.x - min.x) + min.x, Math.random()*(max.y - min.y) + min.y));
			enemies[i].setVelocity(new Vector2(0,Math.random()*speed + speed));
		}
	}

	// returning false means to be deleted
	public boolean render()
	{
		boolean res = false;

		for (int i=0; i < numEnemies; i++)
		{
			res = enemies[i].render() || res;
		}

		return res;
	}

	// returning false will result in a regeneration of formation
	public void update()
	{
		for (int i=0; i < numEnemies; i++)
		{
			enemies[i].update();
		}
	}

	public boolean hit(BoundingBox2 bb)
	{
		boolean res = false;

		for (int i=0; (i < numEnemies) && !res; i++)
		{
			res = enemies[i].hit(bb);
		}

		return res;
	}

	public boolean sphereHit(float radius, Vector2 center)
	{
		boolean res = false;

		for (int i=0; i < numEnemies; i++)
		{
			res = enemies[i].sphereHit(radius, center) || res;
		}

		return res;
	}

} // end RandomFormation
