//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

/*
	FIXME!
	There are a lot of things to optimize in this class ex:

	* There are a few number constants that should be declaired as constants
*/

package starship2d;

import org.lwjgl.Display;
import org.lwjgl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.Window;
import org.lwjgl.opengl.glu.Cylinder;
import org.lwjgl.opengl.glu.Disk;
import org.lwjgl.opengl.glu.GLU;
import org.lwjgl.opengl.glu.PartialDisk;
import org.lwjgl.opengl.glu.Sphere;
import org.lwjgl.input.Keyboard;

public class Particle
{
	// VARIABLES
	private Vector2 pos = new Vector2(0,0);
	private Vector2 size = new Vector2(0,0);;
	private Vector2 vel = new Vector2(0,0);;
	private Vector3 color = new Vector3(0,0,0);;
	private double life;
	private double decLife;
	private float decSpeed = 0;
	private int texture;
	
	Particle()
	{
	}
	
	Particle(String path)
	{
		setTexture(path);
	}
	
	Particle(String path, Vector2 vpos, Vector2 vsize, Vector2 v, Vector3 col, float l, float decL)
	{
		setTexture(path);
		pos = vpos;
		size = vsize;
		vel = v;
		color = col;
		life = l;
		decLife = decL;
	}
	
	public void		setTexture(String path)		{ texture = Texture.loadTexture(path); }
	
	public Vector2	getPos()					{ return pos; }
	public void		setPos(Vector2 newPos)		{ pos = new Vector2(newPos); }
	
	public Vector2	getSize()					{ return size; };
	public void		setSize(Vector2 newSize)	{ size = new Vector2(newSize); }
	
	public Vector2	getVelocity()				{ return vel; };
	public void		setVelocity(Vector2 v)		{ vel = new Vector2(v); }
	
	public Vector3	getColor()					{ return color; };
	public void		setColor(Vector3 col)		{ color = new Vector3(col); }
	
	public double	getLife()					{ return life; }
	public void		setLife(float l)			{ life = l; }
	
	public double	getDecLife()				{ return decLife; }
	public void		setDecLife(float l)			{ decLife = l; }
	
	public float	getDecSpeed()				{ return decSpeed; }
	public void		setDecSpeed(float s)		{ decSpeed = s; }
	
	/*
	* Renders the particle.
	* Expects the blending has been turned on
	* and the correct blend function has been set
	*
	* returns false if the particles life is 0
	*/
	public boolean render()
	{
		GL11.glPushMatrix();
		
		GL11.glTranslatef(pos.x, pos.y, 0);
		
		GL11.glBindTexture(GL11.GL_TEXTURE_2D, texture);
		GL11.glColor4f(color.x, color.y, color.z, (float)life);
		GL11.glBegin(GL11.GL_QUADS);
			GL11.glTexCoord2f(0,0);	GL11.glVertex2f(0, 0);
			GL11.glTexCoord2f(0,1);	GL11.glVertex2f(0, size.y);
			GL11.glTexCoord2f(1,1);	GL11.glVertex2f(size.x, size.y);
			GL11.glTexCoord2f(1,0);	GL11.glVertex2f(size.x, 0);
		GL11.glEnd();
		GL11.glColor3f(1,1,1);

		GL11.glPopMatrix();
		
		life -= decLife * Timer.frameInterval;
		pos.x += vel.x * (Timer.frameInterval * 500);
		pos.y += vel.y * (Timer.frameInterval * 500);
        vel.x -= vel.x * decSpeed * (Timer.frameInterval * 300);
        vel.y -= vel.y * decSpeed * (Timer.frameInterval * 300);

		// is the particle dead ?
		if (life < 0.0)
			return false;
			
		return true;
	}
	
} // end Particle
