//////////////////////////////////////////////////////////////////////////////////////
/*
Author: Henrik Krysell
E-mail: drx@drx.dk
Website: www.drx.dk

Copyright (C) 2004  Henrik Krysell

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//////////////////////////////////////////////////////////////////////////////////////

/*
	FIXME!
	There are a lot of things to optimize in this class ex:

	* I create new Vectors, BoundingBox2 and other objects when i could pre-compute it
	* There are a few number constants that should be declaired as constants
	* Screen width and height needs to be GLOBAL!

	* Organize the update() method better
*/

package starship2d;

import org.lwjgl.Display;
import org.lwjgl.DisplayMode;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.Window;
import org.lwjgl.opengl.glu.Cylinder;
import org.lwjgl.opengl.glu.Disk;
import org.lwjgl.opengl.glu.GLU;
import org.lwjgl.opengl.glu.PartialDisk;
import org.lwjgl.opengl.glu.Sphere;
import org.lwjgl.input.Keyboard;


public class Player extends Object
{
	// VARIABLES
	// FIXME! these should be dependent on the timers resolution
	private static final float BULLET_REPEAT = 350000.0f;
	private static final int BULLET_DAMAGE = 50;
	private static final float ROCKET_REPEAT = 1000000.0f;
	private static final int NEW_ROCKET_SCORE_SCALE = 1000;
	
	private static final double SHIELD_REGENERATION = 1;
	private static final int MAX_SHIELD = 100;
	private static final int NUM_PARTICLES = 20;
	private static final Vector2 BULLET_VEL = new Vector2(0,-0.5f);
	private static final int NUM_ROCKETS = 4;
	
	private int scrWidth = 640;
	private int scrHeight = 480;
	
	private int txtShip;
	private int txtMask;
	private Vector2 pos = new Vector2(0,0);
	private Vector2 size = new Vector2(35,35);
	private Vector2 v = new Vector2(0,0);
	private float decSpeed = 0.008f;
	private boolean sp = false;
	private boolean zDown = false;
	private Particle bullets[] = new Particle[NUM_PARTICLES];
	private int numBullets = 0;
	private long lastBulTime = 0;
	private long lastRocketTime = 0;
	private ParticleRocket pRocket = new ParticleRocket();
	private ParticleExplosion pExplosion = new ParticleExplosion(30);
	private Rocket rockets[] = new Rocket[NUM_ROCKETS];
	private int numRockets = 0;
	private int rocketCount = 0;
	private boolean isHit = false;
	private Layer layer;
	private GameMenu menu;
	private float sqrRadius = 0;
	private long score = 0;
	private long lastNewRocketScore = 0;
	private OGLFont font = new OGLFont();
	private double shield = MAX_SHIELD;
	private float hitColorFade = 1.0f;
	
	// FUNCTIONS
	Player(Layer lay, GameMenu men) 
	{
		menu = men;
		layer = lay;
		
		txtShip = Texture.loadTexture("starship2d/Data/starship.png");
		txtMask = Texture.loadTexture("starship2d/Data/starshipmask.png");
		
		for (int i=0; i < NUM_PARTICLES; i++)
		{
			bullets[i] = new Particle("starship2d/Data/particle.png",new Vector2(-100,-100), new Vector2(20.0f,20.0f),
										new Vector2(0,0), new Vector3(1.0f,0,0), 0.9f, 0.0f);
		}

		for (int i=0; i < NUM_ROCKETS; i++)
		{
			rockets[i] = new Rocket(layer, this);
			rockets[i].setVelocity(new Vector2(0,0));
			rockets[i].setPos(new Vector2(100,100));
		}
		
		pRocket.init(50,"starship2d/Data/particle.png");

		sqrRadius = size.x*size.x / 4 + size.y*size.y / 4;

		// position the player at the center
		pos = new Vector2(scrWidth/2, scrHeight/2);
	
		font.buildFont("Courier New",24);
	}

	protected void finalize() throws Throwable 
	{
		try 
		{
			System.out.println("Player score was: " + score);
    	} 
		finally 
		{
			super.finalize();
		}
	}

	public Vector2 getPos()			{ return new Vector2(pos); }
	public void addScore(int s)		{ score += s; }
	public long getScore()			{ return score; }
	public OGLFont getFont()		{ return font; }
	public void init()				{}

	public boolean render()
	{
		boolean res = true;
		
		// draw score, rocket count and lifes
		font.print("Score: " + score, new Vector2(0,450));
		font.print("Rockets: " + (NUM_ROCKETS - rocketCount), new Vector2(200,450));
		font.print("Shield: " + (int)shield, new Vector2(400,450));

		// render the rockets first
		for (int i=0; i < numRockets; i++)
		{
			if (!rockets[i].render())
			{
				Rocket r = rockets[i];
				rockets[i] = rockets[numRockets-1];
				rockets[numRockets-1] = r;
				
				numRockets--;
			}
		}

		GL11.glEnable(GL11.GL_BLEND);
		
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA,GL11.GL_ONE);
		pRocket.render();

		if (isHit)
		{
			res =  pExplosion.render();
		}
		else
		{
			GL11.glPushMatrix();
			
			GL11.glColor3f(1.0f,hitColorFade,hitColorFade);
			GL11.glTranslatef(pos.x,pos.y,0);
			GL11.glRotatef(v.x*50,0,1,0);
			
			// draw the mask
			GL11.glBlendFunc(GL11.GL_DST_COLOR,GL11.GL_ZERO);
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, txtMask);
			GL11.glBegin(GL11.GL_QUADS);
				GL11.glTexCoord2f(0,0);	GL11.glVertex2f(0, 0);
				GL11.glTexCoord2f(0,-1);	GL11.glVertex2f(0, size.y);
				GL11.glTexCoord2f(-1,-1);	GL11.glVertex2f(size.x, size.y);
				GL11.glTexCoord2f(-1,0);	GL11.glVertex2f(size.x, 0);
			GL11.glEnd();
			
			// draw the texture
			GL11.glBlendFunc(GL11.GL_ONE, GL11.GL_ONE);
			GL11.glBindTexture(GL11.GL_TEXTURE_2D, txtShip);
			GL11.glBegin(GL11.GL_QUADS);
				GL11.glTexCoord2f(0,0);	GL11.glVertex2f(0, 0);
				GL11.glTexCoord2f(0,-1);	GL11.glVertex2f(0, size.y);
				GL11.glTexCoord2f(-1,-1);	GL11.glVertex2f(size.x, size.y);
				GL11.glTexCoord2f(-1,0);	GL11.glVertex2f(size.x, 0);
			GL11.glEnd();
			
			GL11.glColor3f(1.0f,1.0f,1.0f);
			GL11.glPopMatrix();
		}

		// render the bullets
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA,GL11.GL_ONE);
		for (int i=0; i < numBullets; i++)
		{
			bullets[i].render();
		}

		GL11.glDisable(GL11.GL_BLEND);

		return res;
	}
	
	public void update()
	{
        Keyboard.poll();
		// if we are hit this function is not to be evaluated
		if (isHit)
		{
			if (Keyboard.isKeyDown(Keyboard.KEY_TAB))
			{
				// FIXME! rest all variables here
				isHit = false;
			}
			else
				return;
		}
		
		float dt = Timer.frameInterval * 1.5f;
        if(Keyboard.isKeyDown(Keyboard.KEY_UP) || Keyboard.isKeyDown(Keyboard.KEY_E))
        {
            v.y -= dt;
        }
        if(Keyboard.isKeyDown(Keyboard.KEY_DOWN) || Keyboard.isKeyDown(Keyboard.KEY_D))
        {
            v.y += dt;
        }
        if(Keyboard.isKeyDown(Keyboard.KEY_RIGHT) || Keyboard.isKeyDown(Keyboard.KEY_F))
        {
            v.x += dt;
        }
        if(Keyboard.isKeyDown(Keyboard.KEY_LEFT) || Keyboard.isKeyDown(Keyboard.KEY_S))
        {
            v.x -= dt;
        }
        if(!Keyboard.isKeyDown(Keyboard.KEY_SPACE))
        {
            sp = false;
        }
        if(Keyboard.isKeyDown(Keyboard.KEY_SPACE) && !sp)
        {
            sp = true;
        }
        if( Keyboard.isKeyDown(Keyboard.KEY_Z) && !zDown)
        {
			zDown = true;
		}
        if(!Keyboard.isKeyDown(Keyboard.KEY_Z))
        {
            zDown = false;
        }

		// Fire rocket
		if (zDown && (Timer.frameTime - lastRocketTime) > ROCKET_REPEAT)
		{
			lastRocketTime = Timer.frameTime;
			if (rocketCount < NUM_ROCKETS && numRockets < NUM_ROCKETS)
			{
				rockets[numRockets].setVelocity(new Vector2(0,-0.1f));
				rockets[numRockets].setPos(new Vector2(pos.x - 15, pos.y));
				rockets[numRockets].reset();
				
				numRockets++;
				rocketCount++;
			}
		}
		
		// fire bullet
		if (sp && (Timer.frameTime - lastBulTime) > BULLET_REPEAT && (numBullets <  NUM_PARTICLES))
		{
			lastBulTime = Timer.frameTime;
			bullets[numBullets].setPos(new Vector2(pos.x + 8, pos.y));
			bullets[numBullets].setVelocity(new Vector2(0, BULLET_VEL.y));

			// this if statement should always be true but just in case
			if (numBullets + 1 <= NUM_PARTICLES)
				numBullets++;
			else
				System.out.println("ERROR bullets out of bounds");
		}

        pos.x += v.x * (Timer.frameInterval * 500);
        pos.y += v.y * (Timer.frameInterval * 500);
        v.x -= v.x * decSpeed * (Timer.frameInterval * 300);
        v.y -= v.y * decSpeed * (Timer.frameInterval * 300);
        
		if (hitColorFade < 1.0f)
			hitColorFade += 3*Timer.frameInterval;
			
		// check collision
		if (!isHit && layer.checkCollision(new BoundingBox2(pos, size), this))
		{
			isHit = true;
			pExplosion.init(1.0f, 0.5f, 0.1f, decSpeed, pos, new Vector2(20,20), v);
			pRocket.setRegenerate(false);
			menu.setEnd(true);
		}
        
		pos.x = (pos.x > (scrWidth - size.x)) ? (scrWidth - size.x) : pos.x;
        pos.y = (pos.y > (scrHeight - size.y)) ? (scrHeight - size.y) : pos.y;
        pos.x = (pos.x < 0) ? 0 : pos.x;
        pos.y = (pos.y < 0) ? 0 : pos.y;
		
		pRocket.setCurVel(v);
		pRocket.setCurPos(new Vector2(pos.x + 12, pos.y + 33));
		
		// update bullets
		for (int i=0; i < numBullets; i++)
		{
			Vector2 p = bullets[i].getPos();
			Vector2 s = bullets[i].getSize();

			BoundingBox2 bb = new BoundingBox2(p,s);
			if (p.y < 0 || layer.checkCollision(bb,this))
			{
				Particle b = bullets[i];
				bullets[i] = bullets[numBullets-1];
				bullets[numBullets-1] = b;

				numBullets--;
			}
		}
		
		// when you have killed enough enemies you get a new rocket, check for this here
		if ((score >= lastNewRocketScore + NEW_ROCKET_SCORE_SCALE) && rocketCount > 0)
		{
			rocketCount--;
			lastNewRocketScore = score;
		}
		
		// regenerate the shield
		if (shield < MAX_SHIELD)
			shield += SHIELD_REGENERATION * Timer.frameInterval;
	}

	public void destroy()
	{
	}
	
	public boolean hit(BoundingBox2 bb) 
	{
		if (isHit)
			return false;
		
		// FIXME! optimize this, don't create a new BoundingBox2 instance every time
		boolean res = bb.boxInBox(new BoundingBox2(pos,size));

		if (res)
		{
			shield -= BULLET_DAMAGE;
			hitColorFade = 0.0f;
			
			isHit = shield < 0.001;
		}
		
		if (isHit)
		{
			pExplosion.init(1.0f, 0.5f, 0.1f, 0, pos, new Vector2(20,20), v);
			pRocket.setRegenerate(false);
			menu.setEnd(true);
		}
		
		return res;
	}

	public boolean sphereHit(float sqrRad, Vector2 center)
	{
		// FIXME! optimze this
		Vector2 vv = new Vector2( (pos.x + size.x / 2) - center.x, (pos.y + size.y / 2) - center.y);

		float dist = vv.x*vv.x + vv.y*vv.y;
		return dist < sqrRad + sqrRadius;
	}

} // end Player
